'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = function (ComponentStyle) {
  var createStyledComponent = function createStyledComponent(target, rules, props) {
    var isValidTarget = target && typeof target !== 'string';
    var prevProps = {};

    if (isValidTarget) {
      prevProps = (typeof target === 'undefined' ? 'undefined' : _typeof(target)) === 'object' ? target.props : typeof target === 'function' ? target.options.props : {};
    }

    var mergedProps = Object.assign({ value: {} }, prevProps, props);
    var componentStyle = new ComponentStyle(rules);

    return {
      props: mergedProps,
      data: function data() {
        return { localValue: this.value };
      },

      watch: {
        value: function value(newVal) {
          this.localValue = newVal;
        },
        localValue: function localValue() {
          this.$emit('input', this.localValue);
        }
      },
      render: function render(createElement) {
        var self = this;
        return createElement(target, {
          class: [self.generatedClassName],
          props: self.$props,
          domProps: {
            value: self.localValue
          },
          on: {
            input: function input(event) {
              self.localValue = event.target.value;
            },
            click: function click(event) {
              self.$emit('click', event);
            }
          }
        }, this.$slots.default);
      },

      methods: {
        generateAndInjectStyles: function generateAndInjectStyles(style, props) {
          return style.generateAndInjectStyles(props);
        }
      },
      computed: {
        generatedClassName: function generatedClassName() {
          var componentProps = Object.assign({}, this.$props);
          return this.generateAndInjectStyles(componentStyle, componentProps);
        }
      },
      extend: function extend() {
        for (var _len = arguments.length, extendedRules = Array(_len), _key = 0; _key < _len; _key++) {
          extendedRules[_key] = arguments[_key];
        }

        var extended = [];

        extendedRules[0].forEach(function (line, key) {
          extended.push(line);
          extended.push(extendedRules[key + 1]);
        });

        return createStyledComponent(target, rules.slice().concat(extended), mergedProps);
      },
      withComponent: function withComponent(newTarget) {
        return createStyledComponent(newTarget, rules, mergedProps);
      }
    };
  };
  return createStyledComponent;
};

module.exports = exports['default'];